!**************************************************************************************************
! CO2 concentration
! Note: this subroutine will be removed when GCM was connected
!**************************************************************************************************
SUBROUTINE co2_density (LAT)

!_____________ Set variables
!Namespace
   USE data_structure
   USE time_counter
   USE grid_status_current2
   implicit none
   
!Augments
   real   ,intent(IN) ::LAT    !latitude  (degree)
   
!Local variables
!   real x                                      !for temporal usage
!   real a0, a1, a2, a3, a4, a5, a6, a7, a8, a9 !for temporal usage
   
!_____________ Main part
   co2atm = 350.0
   
!!_____________ Atmospheric CO2  conditions [Call cd_trend]
!   !year of examine (A.D. from 1953)
!   x   = 1990.0
!   !annual mean increase, a6
!   a0 = 1904299.0
!   a1 = -3322.4242
!   a2 = 1.6541596
!   a3 = 1.3362655  /10000.0
!   a4 = -3.0828809 /10000000.0
!   a5 = 6.2121261  /100000000000.0
!    a6 = a0 + x * (a1 + x * (a2 + x * (a3 + x * (a4 + x * a5))))
!   !increment according to the year, a7 (constant CO2)
!   a7   = 0.0 ! (-52.11+0.026984*x)*(DBLE(doy)-5.5)/12.0
!   !north-south gradient, a8
!   a8   = 1.6*((LAT)/85.0)
!   !seasonal oscillation, a9
!   if ((LAT)>=0.0) then
!      a9 = exp(0.04*(LAT)) / 2.0*sin( (real(doy-  1.0))/365.0*2.0*PI )
!   else 
!      a9 = exp(0.04*(LAT)) / 2.0*sin( (real(doy+181.0))/365.0*2.0*PI )
!   end if
!   !sumup
!   co2atm = a6 + a7 + a8 + a9   !background CO2 concentration
   
END SUBROUTINE co2_density



!**************************************************************************************************
! Air properties
!**************************************************************************************************
SUBROUTINE air (tmp_air, humid, wind, ALT)

!_____________ Set variables
!Namespace
   USE data_structure
   USE grid_status_current2
   implicit none
   
!Augments
   real,intent(IN) ::tmp_air   !2m air temperature (Celcius)
   real,intent(IN) ::humid     !specific humidity (kg  kg-1)
   real,intent(IN) ::wind      !wind velocity (m s-1)
   real,intent(IN) ::ALT       !altitude (m above MSL)
   
!Local variables
   real a1, a2  !for temporal memory
   
!_____________ Main part
!ap: air pressure (hPa)
   ap = 1013.25 * exp ( -0.2838472*ALT /(8.3144*(tmp_air+ZAT)) )
   
!vp: vapour pressure (hPa)
   vp = ap * humid / ( 0.622 + 0.378 * humid )
   
!vp_sat: saturated vapour pressure (hPa)
   if(tmp_air > 0.0)then   !@water surface
      vp_sat = 6.1078*(10.0**((7.5*tmp_air)/(237.3+tmp_air)))
   else                    !@ice surface
      vp_sat = 6.1078*(10.0**((9.5*tmp_air)/(265.3+tmp_air)))
   endif
   vp_sat = max(0.0, vp_sat)
   
!vpd: vapour pressure deficit (hPa)
   vpd = max(0.0, vp_sat - vp)
   
!slope_vps: slope of saturated vapour pressure as a function of temperature
   if(tmp_air > 0.0)then  !  at  water  surface
      a1 = 6.1078 * (2500.0-2.4*tmp_air)
      a2 = ( 10.0**((7.5*tmp_air)/(237.3+tmp_air)) )
   else                 !  at  ice  surface
      a1 = 6.1078 * 2834.0
      a2 = (10.0**((9.5*tmp_air)/(265.3+tmp_air)))
   endif
   slope_vps = ( a1 / (0.4615*(ZAT+tmp_air)*(ZAT+tmp_air)) ) * a2
   
!dnsa: air density
   dnsa = 1.293 * (ZAT / (tmp_air+ZAT)) * (ap / 1013.25) * (1.0 - 0.378 * vp / ap)
   
END SUBROUTINE air



!**************************************************************************************************
! Radiation properties
!**************************************************************************************************
SUBROUTINE radiation (LAT, cloud)

!_____________ Set variables
!Namespace
   USE data_structure
   USE time_counter
   USE grid_status_current2
   USE grid_status_record
   implicit none
   
!Augments
   real   ,intent(IN)::LAT        !latitude  (degree)
   real   ,intent(IN)::cloud      !total cloudness (fraction)
   
!Local variables
   real rad_stratosphere !shortwave radiation at the stratosphere (W/m2)
   real sl_dec           !solar  declination (degree)
   real ha               !angle from sun-rise to median passage (degree)
   real rad_diffuse      !diffused radiation in rad (W/m2)
   real rad_direct       !direct radiation in rad (W/m2)
   
   real x                !for general usage
   real a1, a2           !for general usage
   
!_____________ Main part
!sl_dec: solar declination of doy (degree)
   sl_dec = 23.45 * sin(DtoR * 360.0 * ( real(doy) - 81.0 ) / 365.0)
   
!sl_hgt: solar hight at midday (degree)
   x      = sin(LAT * DtoR) * sin(sl_dec * DtoR) + &
            cos(LAT * DtoR) * cos(sl_dec * DtoR)
   x      = min(1.0, max(-1.0, x))
   sl_hgt = asin(x) * RtoD
   
!trap for high latitude regions
if (sl_hgt<=0.1) then
   dlen        = 0.0
   rad         = 0.0
   par         = 0.0
   par_diffuse = 0.0
   par_direct  = 0.0
   return
endif

!dlen: day length (hour)
   x      = -tan(LAT * DtoR) * tan(sl_dec * DtoR)
   ha     = RtoD * acos( min(1.0, max(-1.0, x)) ) !angle from sun-rise to median passage
   dlen   = 2.0 * (ha / 15.0)
   
!rad_stratosphere: shortwave radiation at the atmosphere-top (W/m2) 
    a1 = DtoR * 360.0 * (doy/365.0) !seasonal angle of the earth's orbit (radian)
    a2 = 1.00011 + 0.034221 * cos(a1) + 0.00128 * sin(a1) + &
         0.000719 * cos(2.0*a1) + 0.000077 * sin(2.0*a1)
    rad_stratosphere = max(0.0, 1367.0 * sin(sl_hgt * DtoR) * a2)
    
!rad: global radiation at the ground surface (W/m2)
   x = 0.803 - 0.34*cloud - 0.458*cloud*cloud  !Black's
   x = 0.8964 - 0.5392 * cloud   !new regression based on NCEP/NCAR data
   x = min(1.0,  max(0.0, x))
   rad = rad_stratosphere * x
   
!par: photosynthetically active radiation in mid_day (micro mol photon m-2 s-1)
!   based  on  the  empirical  Tooming's  equation
   if(rad_stratosphere /= 0.0) then
      rad_diffuse = max( 0.0 , rad * (0.958 - 0.982*(rad/rad_stratosphere)) )
      rad_direct  = max( 0.0 , rad - rad_diffuse )
      
      par_diffuse = 4.2 * 0.57 * rad_diffuse
      par_direct  = 4.6 * 0.43 * rad_direct 
      par         = par_diffuse + par_direct
   else
      par         = 0.0
      par_diffuse = 0.0
      par_direct  = 0.0
   endif
   
END SUBROUTINE radiation



!**************************************************************************************************
! Net radiation
!**************************************************************************************************
SUBROUTINE net_radiation (tmp_air, cloud, Albedo_soil0)

!_____________ Set variables
!Namespace
   USE data_structure
   USE vegi_status_current1
   USE vegi_status_current2
   USE grid_status_current1
   USE grid_status_current2
   implicit none
   
!Augments
   real,intent(IN)::tmp_air      !2m air temperature (Celcius)
   real,intent(IN)::cloud        !total cloudness, fraction
   real,intent(IN)::Albedo_soil0 !albedo, default
   
!Local variables
   real    radnet_long  !net longwave radiation (W/m2)
   real    ir           !interception by leavies
   
!_____________ Main part
!albedo_soil: soil surface albedo
   albedo_soil = Albedo_soil0 + (0.7-Albedo_soil0) / (1+exp(-0.05*(pool_snow-70.0)))
   albedo_soil = max(0.05, min(0.70, albedo_soil))

!albedo_veg: canopy albedo (temporal)
   select case (biome)
      case (3:9)    !forests
         albedo_veg = 0.15
      case (10)     !larch forests (from Fukuda personal observation)
         albedo_veg = 0.107
      case default  !grass land, desert, etc
         albedo_veg = 0.24
   end select

!radnet_long: net longwave radiation
   radnet_long = (5.6703/100000000.0) * ((tmp_air + ZAT)**4.0) &
                  * (0.39 + 0.058/(vp+1.0)) * (1.0 - 0.65*cloud)

!ir: interception by canopy
   !by tree crown
   ir = min(1.0, par_grass / max(0.01, par) )
   
   !by grass layer
   if (type_grass==3) then
      ir = ir * exp( -1.0 * ( 0.9*lai_grass(1)*eK(C3g_no) + 0.1*lai_grass(2)*eK(C4g_no) ) )
   else
      ir = ir * exp( -1.0 * ( 0.1*lai_grass(1)*eK(C3g_no) + 0.9*lai_grass(2)*eK(C4g_no) ) )
   end if
   ir = min(1.0, max(0.0, ir) )
   
!radnet_veg: net radiation of plant canopy
   radnet_veg   = ( rad*(1.0-albedo_veg) - radnet_long ) * (1.0 - ir_tree * ir_grass)
   radnet_veg   = max (0.0, radnet_veg)
   
!radnet_soil: net radiation of soil surface
   radnet_soil   = ( rad*(1.0-albedo_soil) - radnet_long ) * ir_tree * ir_grass
   radnet_soil   = max (0.0, radnet_soil)
   
END SUBROUTINE net_radiation



!**************************************************************************************************
! Water budget module, daily computation
!**************************************************************************************************
SUBROUTINE waterbudget (tmp_air, tmp_soil, prec, wind, W_fi, W_wilt, W_sat, W_mat)

!_____________ Set variables
!Namespace
   USE data_structure
   USE time_counter
   USE vegi_status_current1
   USE grid_status_current1
   USE grid_status_current2
   USE grid_status_record
   implicit none
   
!Augments
   real,intent(IN)              ::tmp_air  !air temperature (Celcius)
   real,dimension(30),intent(IN)::tmp_soil !ground surface temperature (Celcius)
   real,intent(IN)              ::prec     !precipitation,  mm day-1
   real,intent(IN)              ::wind     !wind velocity (m s-1)
   
   real,intent(IN)::W_fi     !filed capacity   (m3/m3, 0.0 -> 1.0)
   real,intent(IN)::W_wilt   !wilting point    (m3/m3, 0.0 -> 1.0)
   real,intent(IN)::W_sat    !saturate point   (m3/m3, 0.0 -> 1.0)
   real,intent(IN)::W_mat    !matrix potential
   
!Local variables
   real flux_snow  !snow fall (mm/day)
   real flux_rain  !rain fall (mm/day)
   real flux_ic    !intercepted rain (mm/day)
   real flux_tw    !snow thaw water (mm/day)
   real flux_sfc   !available water on soil surface (mm/day)
   real flux_ev    !evaporation flux (mm/day)
   real flux_tr    !transpiration flux (mm/day)
   real flux_ev_pm !potential evaporation rate  (mm/day)
   real flux_tr_pm !potential transpiration rate  (mm/day)
   
   real lai        !leaf area index (m2 m-2)
   real rain_no    !expected number of rain event (day-1)
   
   real c_aero     !aerodynamic conductance (s-1 m)
   real c_soil     !soil conductance        (s-1 m)
   real c_leaf     !canopy conductance      (s-1 m)
   
   real h_veg               !height of vegetaiton surface (m)
   real pool_total          !for checker (total water available in this virtual field)
   
   real    water_available
   real    x, y, a, b, c    !for general usage
   integer layer, no        !for counter
   
!_____________ initialize daily hydrological variables
   hyd_ro = 0.0
   hyd_ic = 0.0
   hyd_ev = 0.0
   hyd_tr = 0.0
   
!_____________ checker
   pool_total  = prec + pool_snow + sum(pool_w(:))
   
!_____________ Calculate Potential Eveporation & Transpiration
   
   !*** Preparetion for Evapotranspiration
   !c_aero: aerodynamic conductance (s m-1)
   c_aero = (1+0.537*wind) / 250.08
   
   !c_soil: surface soil conductance (s m-1)
   x = pool_w(1) / (W_fi*Depth) ; x = min(1.0, x)
   c_soil = 0.0015 * (x**2.0) !Coefficient -> Just Only Parametarization
   
   !c_leaf: canopy conductance (m s-1)
   c_leaf = 0.0224 * canopy_cond !1 mol gas = 0.0224 m3 at 0C and 1atm
   
   !*** Potential transpiration
   if (c_leaf>0.0001) then
      !a: sum of daily netradiation (J m-2 day-1)
      a = 0.5 * radnet_veg           * dlen * 3600 / (2454*1000)
      b = dnsa * 1012 * vpd * c_aero * dlen * 3600 / (2454*1000)
      c = slope_vps + 0.667 * (1.0+c_aero/c_leaf)
      
      flux_tr_pm = ( a*slope_vps + b ) / c
      !     0.5             = fraction of daily mean radiation to midday radiation
      !     3600   (s hr-1) = secound -> hour]
      !     2454   (J  g-1) = heat of vaporization at 20C
      !     1000   (g mm-2) = 1mm equivalent water for evaporation
   else
      flux_tr_pm = 0.0
   endif
   
   !*** Potential evaporation
   if (c_soil>0.0001) then
      !a: sum of daily netradiation (J m-2 day-1)
      a = 0.5 * radnet_soil          * dlen * 3600 / (2454*1000)
      b = dnsa * 1012 * vpd * c_aero *   24 * 3600 / (2454*1000)
      c = slope_vps + 0.667 * (1.0+c_aero/c_soil)
      
      flux_ev_pm = ( a*slope_vps + b ) / c
   else	
      flux_ev_pm = 0.0
   endif
   
!_____________ Fluxus of snow, rain, and thaw water
   !x: proportion of snow in precipitation
   x = 1.0 / ( 1.0 + exp(0.75*tmp_air - 1.5) )
   
   !y: proportion of snow melt in this day
   y = 1.0 / ( 1.0 + exp( -0.3*(tmp_soil(1)-10.0) ) )
   
   !water balance
   flux_snow   = prec * x
   flux_rain   = prec * (1.0-x)
   flux_tw     = pool_snow  * y
   pool_snow   = pool_snow + flux_snow - flux_tw
   
   !checker #1
   y = flux_rain + flux_tw + pool_snow + sum(pool_w(:))
   if ( Logging==1 .and. abs(pool_total-y)>0.1 ) then
      write(*,'(a,2f7.1)') 'error@waterbudget_1', pool_total, y
   endif
   
!_____________ Canopy Interception
   
   !calculate LAI
   lai = 0.0
   
   do no=1, Max_no
      if (tree_exist(no)) lai = lai + la(no)
   end do
   lai = lai / Max_loc / Max_loc
   
   if ( type_grass == 3 ) then
      lai = lai + 0.9*lai_grass(1) + 0.1*lai_grass(2)
   else
      lai = lai + 0.1*lai_grass(1) + 0.9*lai_grass(2)
   end if
   
   !determine fraction interception
   rain_no = 0.1 * flux_rain
   if ( flux_tr_pm + flux_ev_pm <= 50.0 * 12.0 / 365.0 ) then
      rain_no = min( rain_no,  5.0 * 12.0 / 365.0 )
   else
      rain_no = min( rain_no, 10.0 * 12.0 / 365.0 )
   endif
   
   flux_ic = 3.0 * rain_no * ( 1.0 - exp(-1.0*lai) )
   flux_ic = min(flux_rain, flux_ic)          
   
   !water balance
   !flux_ic: intercepted water (mm)
   hyd_ic   = flux_ic           
   flux_sfc = flux_rain - flux_ic + flux_tw 
   
   !cheker #2
   pool_total = pool_total - flux_ic
   y = flux_sfc + pool_snow + sum(pool_w(:))
   if ( Logging==1 .and. abs(pool_total-y)>0.1 )  then
      write(*,'(a,2f7.1)') 'error@waterbudget_2', pool_total, y
   endif
   
!_____________ Penetration
   Do layer=29, 1, -1
      
      !x: potential penetration water in pesspective of soil properties (mm)
      x = max(0.0, pool_w(layer)-W_fi*Depth )
      
      !soil percolation will be inhibited if lower layer is frozen
      if ( tmp_soil(layer  ) < 0.0 ) x=0.0
      if ( tmp_soil(layer+1) < 0.0 ) x=0.0
      
      !adjust potential maximum water permitted to absorbed by lower soil layer
      x = min(x, W_sat*Depth - pool_w(layer+1))
      x = max(x, 0.0)
      
      !water balance
      pool_w(layer  ) = pool_w(layer  ) - x
      pool_w(layer+1) = pool_w(layer+1) + x
      
   End do
   
   !cheker #3
   y = flux_sfc + pool_snow + sum(pool_w(:))
   if ( Logging==1 .and. abs(pool_total-y)>0.1 ) then
      write(*,'(a,2f7.1)') 'error@waterbudget_3', pool_total, y
   endif
   
!_____________ Infiltration: soil_surface -> soil_layer_1
   if (flux_sfc > 0.0) then
      !x: absorbed water
      x = max(0.0, W_sat*Depth - pool_w(1)) !maximum volume of absorbed water
      x = min(flux_sfc, x)                  !actual  volume of absorbed water
      if ( tmp_soil(1) < 0.0 ) x=0.0        !when first soil layer is frozen
      
      !water balance
      pool_w(1)  = pool_w(1)  + x
      flux_sfc   = flux_sfc   - x
      hyd_ro     = hyd_ro     + flux_sfc !recorder
   endif
   
   !cheker #4
   pool_total = pool_total - flux_sfc !checker
   y = pool_snow + sum(pool_w(:))
   if ( Logging==1 .and. abs(pool_total-y)>0.1 )  then
      write(*,'(a,2f7.1)') 'error@waterbudget_4', pool_total, y
   endif
   
!_____________ Actual transpiration (flux_tr)
   flux_tr_pm = max(0.0, flux_tr_pm - flux_ic)
   
   water_available = 0.0
   do layer=1, 10
   if ( tmp_soil(layer) >= 0.0 ) then
      x = pool_w(layer) - Depth*W_wilt
      water_available = water_available + max(x, 0.0)
   endif
   enddo
   
   a       = 0.85 !a = 0.1
   b       = water_available + flux_tr_pm
   c       = water_available * flux_tr_pm
   flux_tr = (b - sqrt( b*b - 4.0*a*c )) / (2.0*a)
   flux_tr = max(flux_tr, 0.0)
   flux_tr = min(flux_tr, water_available)
   
   !** water balance (from top layer)
   x = flux_tr
   do layer=1,10
   if ( tmp_soil(layer) >= 0.0 ) then
      y = max( 0.0, pool_w(layer) - Depth*W_wilt ) !water availbale at the present layer
      y = min (x, y)                               !actual water transpirated from this layer
      pool_w(layer) = pool_w(layer) - y
      x             = x             - y
   endif
   enddo
   
   !water recorder
   hyd_tr = flux_tr
   
   !cheker #5
   pool_total = pool_total - flux_tr
   y = pool_snow + sum(pool_w(:))
   if ( Logging==1 .and. abs(pool_total-y)>0.1 )  then
      write(*,'(a,2f7.1)') 'error@waterbudget_5', pool_total, y
   endif
   
!_____________ Actual evaporation
   a = 0.85 !a = 0.1
   b = pool_w(1) + flux_ev_pm
   c = pool_w(1) * flux_ev_pm
   flux_ev = (b - sqrt( b*b - 4.0*a*c )) / (2.0*a)
   flux_ev = max(flux_ev, 0.0      )
   flux_ev = min(flux_ev, pool_w(1))
   
   !water balance
   pool_w(1) = pool_w(1) - flux_ev
   hyd_ev    = flux_ev
   
   !cheker #6
   pool_total = pool_total - flux_ev
   y = pool_snow + sum(pool_w(:))
   if ( Logging==1 .and. abs(pool_total-y)>0.1 )  then
      write(*,'(a,2f7.1)') 'error@waterbudget_6', pool_total, y
   endif
   
!write (File_no(1),*) &
!( dnsa*1012*vpd*c_aero / (slope_vps+0.667) ) * 24 * 3600 / (2454*1000), &
!0.26*(1+0.537*wind) * vpd * 0.658/(slope_vps+0.658), &
!flux_ev_pm, &
!flux_tr_pm
   
!_____________ Bowen ration
   x           = 0.0018*dlen*(radnet_veg+radnet_soil) !Daily sum of net radiation (M J day-1 m-2)
   y           = 2.454*(hyd_tr+hyd_ev+hyd_ic)         !Latent heat emission (M J day-1 m-2)
   bowen_ratio = max(0.0, x-y) / max(0.000001, y)
   
END SUBROUTINE waterbudget



!*************************************************************************************************
! light interruption index (daily computation)
!*************************************************************************************************
SUBROUTINE ir_index ()

!_____________ Set variables
!Namespace
   USE data_structure
   USE vegi_status_current1
   USE vegi_status_current2
   USE grid_status_current2
   implicit none
   
!Local variables
   real    x, y     !for temporal usage
   integer no, p    !for loop counter
   
!_____________ Main part
   ir_tree   = 0.00
   ir_grass  = 0.00
   par_grass = 0.00
   
IF ( par > 1.0 ) then
   !ir_tree: radiation-interruption-coefficient by tree corwn
   x = 0.0 ; y = 0.0 
   do no = 1, Max_no
      p = pft(no)
      if ( tree_exist(no) .and. phenology(p) ) then
         x = x + la(no) * max(EK0(p),eK(p)) !sum of light attenuation index (direct radiation)
         y = y + la(no) * EK0(p)            !sum of light attenuation index (diffused radiation)
      endif
   enddo
   x = x / Max_loc / Max_loc
   y = y / Max_loc / Max_loc
   
   ir_tree = (par_direct/par)*exp(-1.0*x) + (par_diffuse/par)*exp(-1.0*y)
   ir_tree = min(1.0, max(0.0, ir_tree) )
   
   !par_grass: PAR on top of the grass layer
   par_grass = par * ir_tree
   
   !ir_grass: radiation-interruption-coefficient by grass leaf
   
   !radiation-interruption-coefficient of C3 grass
   x = (par_direct /par)*exp(-1.0*lai_grass(1)* eK(C3g_no)) + &
       (par_diffuse/par)*exp(-1.0*lai_grass(1)*EK0(C3g_no))  
   
   !radiation-interruption-coefficient of C4 grass
   y = (par_direct /par)*exp(-1.0*lai_grass(2)* eK(C4g_no)) + &
       (par_diffuse/par)*exp(-1.0*lai_grass(2)*EK0(C4g_no))  
   
   if (type_grass==3) then
      ir_grass = 0.9*x + 0.1*y
   else
      ir_grass = 0.1*x + 0.9*y
   end if
   ir_grass = min(1.0, max(0.0, ir_grass) )
   
ENDIF

END SUBROUTINE ir_index
